/*
 * The CIP4 Software License, Version 0.1
 *
 *
 * Copyright (c) 2001 The International Cooperation for the Integration of 
 * Processes in  Prepress, Press and Postpress (CIP4).  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        The International Cooperation for the Integration of 
 *        Processes in  Prepress, Press and Postpress (www.cip4.org)"
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "CIP4" and "The International Cooperation for the Integration of 
 *    Processes in  Prepress, Press and Postpress" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact info@cip4.org.
 *
 * 5. Products derived from this software may not be called "CIP4",
 *    nor may "CIP4" appear in their name, without prior written
 *    permission of the CIP4 organization
 *
 * Usage of this software in commercial products is subject to restrictions. For
 * details please consult info@cip4.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE INTERNATIONAL COOPERATION FOR
 * THE INTEGRATION OF PROCESSES IN PREPRESS, PRESS AND POSTPRESS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the The International Cooperation for the Integration 
 * of Processes in Prepress, Press and Postpress and was
 * originally based on software 
 * copyright (c) 1999-2001, Heidelberger Druckmaschinen AG 
 * copyright (c) 1999-2001, Agfa-Gevaert N.V. 
 *  
 * For more information on The International Cooperation for the 
 * Integration of Processes in  Prepress, Press and Postpress , please see
 * <http://www.cip4.org/>.
 *  
 * 
 */

/******************************************************************************
 *                     Copyright 1998 Agfa-Gevaert N.V.                       *
 *                           All rights reserved                              *
 *                                                                            *
 * The material contained herein may not be reproduced in whole or in part,   *
 *        without the prior written consent of Agfa-Gevaert N.V.              *
 *                                                                            *
 ******************************************************************************/

/******************************************************************************
 *	Includes
 ******************************************************************************/ 

#include "RegExp.h"

namespace JDF
{

/******************************************************************************
 *	Forward declarations
 ******************************************************************************/ 


/******************************************************************************
 *	Defines and constants
 ******************************************************************************/ 


/******************************************************************************
 *	Typedefs
 ******************************************************************************/ 


/******************************************************************************
 *	Classes
 ******************************************************************************/ 

/******************************************************************************
 *	Prototypes
 ******************************************************************************/ 


/******************************************************************************
 *	Implementation
 ******************************************************************************/ 

static bool isLetter(char ch)
{
	return ( ('a' <= ch && ch <= 'z') ||
		     ('A' <= ch && ch <= 'Z') );
}

static bool isDigit(char ch)
{
	return ('0' <= ch && ch <= '9');
}

static bool isLetterOrDigit(char ch)
{
	return isLetter(ch) || isDigit(ch);
}

Regexp::Regexp(const WString& exp)
{
	mExp = exp;
}

Regexp::ExpState Regexp::nextstate(int pos)
{
	if (static_cast<unsigned int>(pos) == mExp.length())
		return FINISHED;
	else if (mExp[pos] == '*')
		return MATCHANY;
	else 
		return MATCHCHAR;
}

bool Regexp::match(const WString& s)
{
	int ind =0;
	int len = s.length();
	char ch;
	unsigned int i =0;
	char matchNext;
	ExpState state = nextstate(i);
	bool isOK;

	while (ind < len)
	{
		if (state == MATCHANY)
		{
			if (i == mExp.length()-1)
				return true;
			i++;
			matchNext = (char)mExp[i];
			while (matchNext == '*')
			{
				if (i == mExp.length()-1)
					return true;
				i++;
				matchNext = (char)mExp[i];
			}
			ch = (char)s[ind];
			isOK = isLetterOrDigit(ch);
			while (isOK && ch != matchNext)
			{
				ind++;
				if (ind==len)
					return false;
				ch = (char)s[ind];
				isOK = isLetterOrDigit(ch);
			}
			if (isOK == false && ch != matchNext)
				return false;
			// eat char
			ind++;
			i++;
			state = nextstate(i);

		}
		else if (state == MATCHCHAR)
		{
			matchNext = (char)mExp[i];
			if (s[ind] !=matchNext)
				return false;
			i++;
			ind++;
			state = nextstate(i);
		}
		else if (state == FINISHED)
		{
			return true;
		}
	}
	return true;
}

} // namespace JDF

/* end of file */
